# Supports Organizer.rb
#=begin
#Copyright 2009, Matt666
#All Rights Reserved

#THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, #INCLUDING, WITHOUT LIMITATION, 
#THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

#License:   		Free

#Author:     	Matthieu NOBLET
#Organization:   Matt666
#Name:      		PointTool
#Version:   		1.1
#SU Version:   	6 and 7
#Date:      		02/05/2009
#Description:   	Draws some points
#Usage:      	Menu "plugins/Draw Cpoints".
#				Type "pointtool" in the ruby console to load it manually.
#History:
#	1.100	02/05/2009
#			It does not create a point on an existing point
#			Supports organizer
#			You can now draw just one point
#
#	1.000   01/09/2008
#			First version
#=end

require 'sketchup.rb'

unless file_loaded?(__FILE__)
	$submenu ? (organizer = $submenu) : (organizer = UI.menu("Plugins"))
	organizer.add_item("Draw Cpoints"){ pointtool }
	
	p=Sketchup.find_support_file("Plugins")
	cmd = UI::Command.new("Draw Cpoints"){ pointtool }
	cmd.large_icon = cmd.small_icon = File.join(p, "PointTool", "PointTool.png")
	cmd.tooltip = cmd.status_bar_text = "Draw Cpoints"
	UI::Toolbar.new("CPoint Tool").add_item cmd
end
file_loaded(__FILE__)

class PointTool
	def initialize
		@ip1 = nil
		@ip2 = nil
		@xdown = 0
		@ydown = 0
	end

	def activate
		model=Sketchup.active_model
		if Sketchup.version.to_i<7
			model.start_operation "Draw Cpoints"
		else
			model.start_operation "Draw Cpoints",true
		end
		@ip = Sketchup::InputPoint.new
		@ip1 = Sketchup::InputPoint.new
		@ip2 = Sketchup::InputPoint.new
		@drawn=false
		if Sketchup.active_model.rendering_options["HideConstructionGeometry"]==true 
			Sketchup.active_model.rendering_options["HideConstructionGeometry"]=false 
		end
		self.reset(nil)
	end
  
	def deactivate(view);view.invalidate if @drawn;end
	
	def onCancel(flag, view);self.reset(view);end
  
	def reset(view)
		@state = 0
		Sketchup::set_status_text(("Specify First Cpoint:"), SB_PROMPT)###
		@ip.clear
		@ip1.clear
		@ip2.clear
		if view
			view.tooltip=nil
			view.invalidate if @drawn
		end
		@drawn = false
	end
	
	def onMouseMove(flags, x, y, view)
		if @state==0
			@ip.pick view, x, y
			if @ip != @ip1
				view.invalidate if @ip.display? or @ip1.display?
				@ip1.copy! @ip
				view.tooltip = @ip1.tooltip
			end
			view.tooltip = "First Cpoint:"
		else
			@ip2.pick view,x,y,@ip1
			view.tooltip=@ip2.tooltip if @ip2.valid?
			view.invalidate
			if @ip2.valid?
				length=@ip1.position.distance(@ip2.position)
				Sketchup::set_status_text(length.to_s, SB_VCB_VALUE)###
			end
			view.tooltip="Next Cpoint:"
		end
	end

	def onLButtonDown(flags, x, y, view)
		if @state==0
			@ip1.pick view, x, y
			if @ip1.valid?
				@state = 1
				Sketchup::set_status_text(("Click for Next Cpoint:"), SB_PROMPT)###
				Sketchup::set_status_text(("Length"), SB_VCB_LABEL)###
				@xdown = x
				@ydown = y
			end
		elsif @ip2.valid? && @ip2!=@ip1
			@state = 1
			@ip1.copy! @ip2	
		end
		view.lock_inference
	end

	def onLButtonUp(flags, x, y, view)
		if @ip1.valid?
			self.create_geometry(@ip1.position,view) if self.no_points_here?(@ip1.position)
		elsif @ip2.valid?
			self.create_geometry(@ip2.position,view) if self.no_points_here?(@ip2.position)
		end
	end

	def onKeyDown(key, repeat, flags, view)
		if key == CONSTRAIN_MODIFIER_KEY && repeat == 1
			@shift_down_time = Time.now
			if( view.inference_locked? )
				view.lock_inference
			elsif @state==0 && @ip1.valid?
				view.lock_inference @ip1
				# view.line_width = 3
			elsif @state==1 && @ip2.valid?
				view.lock_inference @ip2, @ip1
				# view.line_width = 3
			end
		end
	end

	def onKeyUp(key, repeat, flags, view)
		view.lock_inference if key==CONSTRAIN_MODIFIER_KEY && view.inference_locked? && (Time.now - @shift_down_time)>0.5
	end

	def onUserText(text, view)
		return if @state == 0
		return if not @ip2.valid?
		begin
			value = text.to_l
		rescue
			puts "Cannot convert #{text} to a Length"
			value = nil
			Sketchup::set_status_text("", SB_VCB_VALUE)###
		end
		return if !value
		pt1 = @ip1.position
		vec = @ip2.position - pt1
		return if vec.length == 0.0
		vec.length = value
		pt2 = pt1 + vec
		self.create_geometry(pt2, view) if self.no_points_here?(pt2)
		@state = 1
		pt2 = view.screen_coords pt2
		@ip1.pick view,pt2[0],pt2[1]
	end
  
	def draw(view)
	    if @ip1.valid?
			if @ip1.display?
				@ip1.draw(view)
				@drawn = true
			end
			if @ip2.valid?
				@ip2.draw(view) if @ip2.display?
				view.set_color_from_line(@ip1, @ip2)
				self.draw_geometry(@ip1.position, @ip2.position, view)
				@drawn = true
			end
		end
	end

	def create_geometry(p,view)
		view.model.active_entities.add_cpoint p
		view.invalidate
	end

	def draw_geometry(pt1, pt2, view)
		view.draw_line(pt1, pt2)
		view.invalidate
	end
	
	def no_points_here?(p)
		Sketchup.active_model.entities.select{|e|e.class==Sketchup::ConstructionPoint}.each do |e|
			return false if e.position.to_a==p.to_a
		end
		return true
	end
end

def pointtool;Sketchup.active_model.select_tool PointTool.new;end

